@@ -4,7 +4,7 @@
 library("mvp")
 
 
-P <- as.mvp("1+x+x^3*z+4x^2*y+ 4*z^4")^3
+P <- as.mvp("1 + x + x^3 z + 4 x^2 y + 4 z^4")^3
 stopifnot(aderiv(P,x=1,y=2,z=3) == aderiv(P,z=3,x=1,y=2))
 
 
